package se.vgregion.portal.myusdissues;

import javax.portlet.PortletPreferences;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.portlet.bind.annotation.ActionMapping;
import org.springframework.web.portlet.bind.annotation.RenderMapping;

/**
 * Controller for edit mode.
 */
@Controller
@RequestMapping("EDIT")
public class MyUsdIssuesEditController {
    private static final Logger LOGGER = LoggerFactory.getLogger(MyUsdIssuesEditController.class);
    /**
     * JSP to show on success.
     */
    public static final String CONFIG_JSP = "preferencesConfig";
    /**
     * JSP to show when fatal error occurred.
     */
    public static final String ERROR_JSP = "fatalError";
    /**
     * Property key name for "Show Link" preference setting.
     */
    public static final String CONFIG_MYUSDISSUES_SHOW_LINK_KEY = "myUsdIssuesShowLinkKey";
    /**
     * PortletPreferences attribute key for list max size.
     */
    public static final String CONFIG_MYUSDISSUES_LIST_SIZE_KEY = "myUsdListLimitKey";
    /**
     * List max size default value.
     */
    public static final int DEFAULT_LIST_SIZE = 5;

    /**
     * Method handling Render request, fetching portlet preferences for view.
     * 
     * @param model
     *            My USD Issues EDIT portlet's ModelMap
     * @param preferences
     *            My USD Issues EDIT portlet's PortletPreferences
     * @return jsp url for view
     */
    @RenderMapping
    public final String showPreferences(final ModelMap model, final PortletPreferences preferences) {
        // Check if save action rendered an error
        if ("true".equals(model.get("saveError"))) {
            return ERROR_JSP;
        }

        // Check if showLinkValue should be checked
        String showLinkValue = preferences.getValue(CONFIG_MYUSDISSUES_SHOW_LINK_KEY, "");
        if ("true".equals(showLinkValue)) {
            model.addAttribute("showLinkValueChecked", "checked");
        }
        // Check list size
        String listSizeValue = preferences.getValue(CONFIG_MYUSDISSUES_LIST_SIZE_KEY, "");
        if (!StringUtils.hasText(listSizeValue)) {
            listSizeValue = String.valueOf(DEFAULT_LIST_SIZE);
        }

        // Set preferences key
        model.addAttribute("showLinkKey", CONFIG_MYUSDISSUES_SHOW_LINK_KEY);
        model.addAttribute("listSizeKey", CONFIG_MYUSDISSUES_LIST_SIZE_KEY);
        model.addAttribute("listSize", listSizeValue);

        return CONFIG_JSP;
    }

    /**
     * Method handling Action request, saving portlet preferences.
     * 
     * @param model
     *            My USD Issues EDIT portlet's ModelMap
     * @param preferences
     *            My USD Issues EDIT portlet's PortletPreferences
     * @param showLink
     *            Setting indicating if USD Issue ID's should have a direct link to USD
     */
    @ActionMapping
    public final void savePreferences(final ModelMap model, final PortletPreferences preferences,
            @RequestParam(value = CONFIG_MYUSDISSUES_SHOW_LINK_KEY, required = false) final String showLink,
            @RequestParam(value = CONFIG_MYUSDISSUES_LIST_SIZE_KEY, required = false) final String listSize) {
        try {
            // Set preference and store value
            preferences.setValue(CONFIG_MYUSDISSUES_SHOW_LINK_KEY, Boolean.valueOf(showLink).toString());
            preferences.setValue(CONFIG_MYUSDISSUES_LIST_SIZE_KEY, listSize);
            preferences.store();
            model.addAttribute("saveError", null);
        } catch (Exception e) {
            LOGGER.error("Error when trying to store My USD Issues preferences.", e);
            model.addAttribute("saveError", "true");
        }
    }
}
