package se.vgregion.portal.myusdissues;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.ResourceBundle;

import javax.portlet.PortletConfig;
import javax.portlet.PortletPreferences;
import javax.portlet.PortletRequest;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.portlet.bind.annotation.RenderMapping;

import se.vgregion.usdservice.USDService;
import se.vgregion.usdservice.domain.Issue;

/**
 * View controller for My USD Issues.
 */
@Controller
@RequestMapping("VIEW")
public class MyUsdIssuesViewController {

    private static final Logger LOGGER = LoggerFactory.getLogger(MyUsdIssuesViewController.class);
    /**
     * JSP to show on success.
     */
    public static final String VIEW_JSP_URL = "myUsdIssues";
    /**
     * JSP to show when fatal error occurred.
     */
    public static final String ERROR_JSP = "fatalError";

    @Autowired
    private PortletConfig portletConfig;

    @Autowired
    private USDService usdService;

    /**
     * Method handling Render request, fetching a list of USD issues for view.
     * 
     * @param model
     *            My USD Issues VIEW portlet's ModelMap
     * @param request
     *            My USD Issues VIEW portlet's RenderRequest
     * @param response
     *            My USD Issues VIEW portlet's RenderResponse
     * @param preferences
     *            My USD Issues VIEW portlet's PortletPreferences
     * @return jsp url for view
     */
    @RenderMapping
    public final String showIssues(final ModelMap model, final RenderRequest request,
            final RenderResponse response, final PortletPreferences preferences) {

        List<Issue> list = new ArrayList<Issue>();

        // Get userId from request
        String userId = getRemoteUserId(request);

        if (StringUtils.hasLength(userId)) {
            Integer listSize = Integer.valueOf(preferences.getValue(
                    MyUsdIssuesEditController.CONFIG_MYUSDISSUES_LIST_SIZE_KEY, "-1"));
            try {
                list = usdService.getRecordsForContact(userId, listSize);
            } catch (Exception e) {
                LOGGER.error("Error when calling USD service", e);
                return ERROR_JSP;
            }
        }

        // Set number of issues in portal title
        ResourceBundle bundle = portletConfig.getResourceBundle(request.getLocale());
        if (bundle != null) {
            response.setTitle(bundle.getString("javax.portlet.title") + " (" + list.size() + ")");
        }

        // Add list and link indicator to model
        model.addAttribute("issueList", list);
        model.addAttribute("showLink",
                preferences.getValue(MyUsdIssuesEditController.CONFIG_MYUSDISSUES_SHOW_LINK_KEY, ""));

        // Return view
        return VIEW_JSP_URL;
    }

    @SuppressWarnings("unchecked")
    private String getRemoteUserId(final PortletRequest request) {
        Map<String, ?> userInfo = (Map<String, ?>) request.getAttribute(PortletRequest.USER_INFO);
        String userId = "";
        if (userInfo != null && userInfo.get(PortletRequest.P3PUserInfos.USER_LOGIN_ID.toString()) != null) {
            userId = (String) userInfo.get(PortletRequest.P3PUserInfos.USER_LOGIN_ID.toString());
        }
        return userId;
    }
}
